﻿using System;
using System.ComponentModel.Composition;
using Granta.HybridBase;

namespace UserModel
{
    #region Source Material Data Definition
    public class SourceData
    {
        [Data("Density", "kg/m^3")]
        public double Density;

        [Data("Young's Modulus", "GPa")]
        public double YoungsModulus;

        [Data("Compressive strength", "MPa")]
        public double CompressiveStrength;

        [Data("Yield strength (elastic limit)", "MPa")]
        public double YieldStrength;
        
        // When requesting price data, the unit takes the form currency/kg:
        [Data("Price", "currency/kg")]
        public double Price;
    }
    #endregion

    [Export("Granta.HybridModel")]
    [Model("Advanced Model (C#)", "A more advanced example model.")]
    [Image("UserModel.model.png")]
    [ModelGroup("Examples", "UserModel.thumbnail.png")]
    public class ExampleModel 
    {
        [Material("Matrix", "Matrix material")]
        [Group("Matrix", 1)]
        public SourceData matrix;

        [RecordName("Matrix", "Abbreviation used in record name")]
        public string _matrixName;

        [Material("Reinforcement", "Reinforcement material")]
        [Group("Reinforcement", 2)]
        public SourceData reinforcement;

        [RecordName("Reinforcement", "Abbreviation used in record name")]
        public string _reinforcementName;

        [Variable("Reinforcement percentage", "%", "Specify the volume fraction")]
        [RangeValues(Start = 10, End = 70, Number = 7, Logarithmic = false)]
        [Bounds(0, 70)]
        [Group("Reinforcement", 3)]
        public double percentage;

        // If we want the user to pick from a list of options, we need to 
        // create an enum to hold the possible choices. 
        // Each option (enumerator) can be marked up with the [OptionItem] attribute, 
        // which takes the description used in the UI combobox.
        public enum LaminateType
        {
            [OptionItem("Unidirectional fiber")]
            Unidirectional,

            [OptionItem("Quasi-isotropic fiber")]
            QuasiIsotropic,
        }

        // The laminateType option that will be used in the calculations.
        // The field must be marked as [Option] and requires a name and a description.
        [Option("Laminate", "Pick a laminate")]
        [Group("Reinforcement", 4)]
        public LaminateType laminate = LaminateType.Unidirectional;

        #region Calculations

        // Density is calculated by calling the method RuleOfMixture:
        [CalculatedData("Density", "kg/m^3")]
        public double Density()
        {
            return RuleOfMixture(reinforcement.Density, matrix.Density);
        }

        // Price - remember the unit should be in currency/kg:
        [CalculatedData("Price", "currency/kg")]
        public double Price()
        {
            // Price is calculated as a rule of mixture, using the density 
            return RuleOfMixture(reinforcement.Density * reinforcement.Price, matrix.Density * matrix.Price) / Density();
        }

        // The Young's Modulus calculation changes depending on the laminateType. 
        // The Unidirectional laminate uses the normal Rule Of Mixture.
        // The QuasiIsotropic laminate calculates a value using the RuleOfMixture, but 
        // with only 0.5 * the reinforcement.
        [CalculatedData("Young's modulus", "GPa")]
        public double YoungsModulus()
        {
            switch (laminate)
            {
                case LaminateType.Unidirectional:
                    return RuleOfMixture(reinforcement.YoungsModulus, matrix.YoungsModulus);

                case LaminateType.QuasiIsotropic:
                    return RuleOfMixture(0.5 * reinforcement.YoungsModulus, matrix.YoungsModulus);

                default:
                    return double.NaN;
            }
        }

        [CalculatedData("Compressive strength", "MPa")]
        public double CompressiveStrength()
        {
            switch (laminate)
            {
                case LaminateType.Unidirectional:
                    {
                        var cs1 = RuleOfMixture(0.75 * reinforcement.CompressiveStrength, matrix.CompressiveStrength);
                        var cs2 = 14.0 * matrix.YieldStrength;
                        return Math.Min(cs1, cs2);
                    }

                case LaminateType.QuasiIsotropic:
                    {
                        var cs1 = RuleOfMixture(0.25 * reinforcement.CompressiveStrength, matrix.CompressiveStrength);
                        var cs2 = 14.0 * matrix.YieldStrength;
                        return Math.Min(cs1, cs2);
                    }

                default:
                    return double.NaN;
            }
        }

        #endregion

        // The Rule Of Mixture implemented as a helper method, and reused in the calculations above.
        private double RuleOfMixture(double a, double b)
        {
            var f = percentage / 100;
            return f * a + (1.0 - f) * b;
        }
    }
}
